﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;
using System.Drawing;

using System.Windows.Forms;

namespace STFXmlParser
{
    public class PageArray
    {
        public Dictionary<string, Field> FieldDict { get; set; }
        public string STFName { get; set; }
        public Dictionary<int, PageScale> ScaleDict { get; set; }
        public Dictionary<int, PageData> PageDict { get; set; }
        public Dictionary<int, PgBMapData> PgBMapDict { get; set; }
        public Dictionary<int, PgZoomData> PgZoomDict { get; set; }

        //public Dictionary<int, List<string>> PageSelDict { get; set; }

        public PageArray()
        {
            _PageArray = new Dictionary<int, List<Field>>();
            FieldDict = new Dictionary<string, Field>();
            STFName = "";
            ScaleDict = new Dictionary<int, PageScale>();
            PageDict = new Dictionary<int, PageData>();
            PgBMapDict = new Dictionary<int, PgBMapData>();
            PgZoomDict = new Dictionary<int, PgZoomData>();

            //PageSelDict = new Dictionary<int, List<string>>();
        }
                
        public int Length()
        {
            return _PageArray.Count;
        }

        public List<Field> GetFieldArrayAt(int index)
        {

            int count = 0;
            List<Field> f = null;

            foreach (KeyValuePair<int, List<Field>> kvp in _PageArray)
            {
                if (count == index)
                {
                    f = kvp.Value;
                    break;
                }

                ++count;
            }

            return f;
        }

        public List<uint> GetBoxedPageList()
        {
            List<uint> pl = new List<uint>();

            // Not returning page 1 [index value 0] if the only box is box1_65535.
            foreach (KeyValuePair<int, List<Field>> kvp in _PageArray)
            {
                if (kvp.Key == 0)
                {
                    List<Field> fa = GetFieldArray(0);

                    if (fa.Count == 1)
                    {
                        if (fa[0].ref_name == "box1_65535")
                        {
                            //MessageBox.Show("Not retrieving page 1 - only box is box1_65535");
                            continue;
                        }
                    }
                }

                pl.Add((uint)(kvp.Key + 1));
            }

            return pl;
        }

        public int GetTotalBoxCount()
        {
            int count = 0;
           
            foreach (KeyValuePair<int, List<Field>> kvp in _PageArray)
                count += kvp.Value.Count;

            return count;
        }

        public List<Field> GetFieldArray(int page)
        {
            List<Field> fa = null;
            
            _PageArray.TryGetValue(page, out fa);

            return fa;
        }



        public void GetFieldArrayRef(int page, ref List<Field> fa)
        {
            _PageArray.TryGetValue(page, out fa);
        }



        public PageScale GetPageScale(int page)
        {
            PageScale f;

            if (ScaleDict.TryGetValue(page, out f) == false)
            {
                f = null;
            }

            return f;
        }

        public PageData GetPageData(int page)
        {
            PageData pd;

            if (PageDict.TryGetValue(page, out pd) == false)
            {
                pd = null;
            }

            return pd;
        }

        public PgBMapData GetPgBMapData(int page)
        {
            PgBMapData bm;

            if (PgBMapDict.TryGetValue(page, out bm) == false)
            {
                bm = null;
            }

            return bm;
        }

        public void GetPgZoomDataRef(int page, ref PgZoomData zd)
        {
            PgZoomDict.TryGetValue(page, out zd);
        }

        public void SetPageScale(int page, double dpi, double xScale, double yScale)
        {
            PageScale ps = null;

            if (ScaleDict.TryGetValue(page, out ps) == true)
            {
                ps.dpi = dpi;
                ps.xScale = xScale;
                ps.yScale = yScale;
            }
            else
            {
                ps = new PageScale();
                ps.dpi = dpi;
                ps.xScale = xScale;
                ps.yScale = yScale;

                ScaleDict.Add(page, ps);
            }
        }

        public void SetPageData(int page, double pageH, double pageW, int pageRot)
        {
            PageData pd = null;

            if (PageDict.TryGetValue(page, out pd) == true)
            {
                pd.pageH = pageH;
                pd.pageW = pageW;
                pd.pageRot = pageRot;
            }
            else
            {
                pd = new PageData();

                pd.pageH = pageH;
                pd.pageW = pageW;
                pd.pageRot = pageRot;

                PageDict.Add(page, pd);
            }
        }

        public void SetPgBMapData(int page, double pgBmapH, double pgBmapW)
        {
            PgBMapData bm = null;

            if (PgBMapDict.TryGetValue(page, out bm) == true)
            {
                bm.pgBmapH = pgBmapH;
                bm.pgBmapW = pgBmapW;
            }
            else
            {
                bm = new PgBMapData();

                bm.pgBmapH = pgBmapH;
                bm.pgBmapW = pgBmapW;

                PgBMapDict.Add(page, bm);
            }
        }

        public void SetPgZoomData(int page, double dpi, double pgBmapH, double pgBmapW)
        {
            /*
             * We do this when the panel(s) that contain the picture box(es) change
             * size such that the picture box image is increased accordingly so we do not
             * see form area around all sides of the image. We call this a dpiAutoDelta.
             * When this happens, we will reset PgZoomData to reflect the new dpi and
             * bitmap height and width for each page and set each page's zoom count to 0
             * and set each page's master and current dpiAutoDelta to false.
             * 
             * We use the dpi and bitmap data when there is a Zoom Reset event so each page
             * will reset back to the original values. We also use them when we are doing zoom
             * in and zoom out event handling paired with the dpiAutoDelta and zoom count settings.
             * 
             * Most of the time these dpi and bitmap settings should be the same, but things change
             * when we have a document with landscape thrown in amongst 'regular' pages, or legal
             * pages, so decided to do it this way for all cases.
            */
            PgZoomData zd = null;

            if (PgZoomDict.TryGetValue(page, out zd) == true)
            {
                zd.zoomCnt = 0;
                zd.dpi = dpi;
                zd.pgBmapH = pgBmapH;
                zd.pgBmapW = pgBmapW;
                zd.mstrDpiAutoDelta = false;
                zd.curDpiAutoDelta = false;
            }
            else
            {
                zd = new PgZoomData();

                zd.dpi = dpi;
                zd.pgBmapH = pgBmapH;
                zd.pgBmapW = pgBmapW;

                PgZoomDict.Add(page, zd);
            }
        }

        public void AddPageField(int page, Field cField)
        {
            List<Field> fa;
            Field f = null;

            if (FieldDict.TryGetValue(cField.ref_name, out f) == false)
                FieldDict.Add(cField.ref_name, cField);

            if (_PageArray.TryGetValue(page, out fa) == true)
            {
                fa.Add(cField);
            }
            else
            {
                fa = new List<Field>();
                fa.Add(cField);

                _PageArray.Add(page, fa);
            }
        }

		// The 'last' field.
        public Field GetBackPageField(int page)
        {
            List<Field> fa;
            Field f = null;
            
            if (_PageArray.TryGetValue(page, out fa) == true)
            {
                f = fa[fa.Count - 1];
            }

            return f;
        }

        public bool WriteXML(string FileName)
        {
            bool success = true;

            string field_xml = "";
            string page_xml = "";
            string form_xml = "";

            
            List<Field> PageFields = null;

            for (int page = 1; page <= _PageArray.Count; ++page)
            {
                if (_PageArray.TryGetValue(page - 1, out PageFields) == true)
                {
                    if (PageFields.Count > 0)
                    {
                        field_xml = "";

                        foreach (Field field in PageFields)
                        {
                            field_xml += field.OutputToXML();
                        }

                        if (field_xml.Length > 0)
                        {
                            page_xml += "<page ref=\"pg" + page + "\">\n";
                            page_xml += field_xml;
                            page_xml += "</page>\n";
                        }
                    }
                }
            }

            if (page_xml.Length > 0)
            {
                // Write the prolog.
                form_xml += "<?xml version=\"1.0\" encoding=\"ISO-8859-1\" standalone=\"no\"?>\n";
                form_xml += "<!DOCTYPE form SYSTEM \"http://cdn.stf.com/dtd/xml_pdf_form_2.dtd\">\n";

                // Write the form element.
                form_xml += "<form ref=\"" + STFName + "\">\n";
                form_xml += page_xml;

                // Write the epilog.
                form_xml += "</form>";

                try
                {
                    byte[] buffer = System.Text.Encoding.UTF8.GetBytes(form_xml);

                    using (FileStream fs = new FileStream(FileName, FileMode.Create, FileAccess.ReadWrite))
                    {
                        fs.Write(buffer, 0, buffer.Length);
                        fs.Close();
                    }

                }
                catch
                {
                    success = false;
                }
            }

            return success;
        }

        private Dictionary<int, List<Field>> _PageArray;
        
    }

    public class FloatRect
    {
        public FloatRect(float left, float top, float right, float bottom)
        {
            this.left = left;
            this.top = top;
            this.right = right;
            this.bottom = bottom;
        }

        public float top { get; set; }
        public float left { get; set; }
        public float bottom { get; set; }
        public float right { get; set; }
    }

    public class Field
    {
        public RectangleF sbox { get; set; }
        public FloatRect bbox { get; set; }
        public string ref_name  {get; set;}
        public string data {get; set;}

        public Boolean isSelected { get; set; }


        public Field()
        {
            data = "";
        }

        public virtual string OutputToXML()
        {
            return "";
        }
    }

    public class PageScale
    {
        public PageScale()
        {
        }

        public double dpi { get; set; }
        public double xScale { get; set; }
        public double yScale { get; set; }
    }

    public class PageData
    {
        public PageData()
        {
        }

        public double pageH { get; set; }
        public double pageW { get; set; }
        public int pageRot { get; set; }
    }

    public class PgBMapData
    {
        public PgBMapData()
        {
        }

        public double pgBmapH { get; set; }
        public double pgBmapW { get; set; }
    }

    public class PgZoomData
    {
        public PgZoomData()
        {
            zoomCnt = 0;
            mstrDpiAutoDelta = false;
            curDpiAutoDelta = false; 
        }

        public int zoomCnt { get; set; }
        public double dpi { get; set; }
        public double pgBmapH { get; set; }
        public double pgBmapW { get; set; }
        public bool mstrDpiAutoDelta { get; set; }
        public bool curDpiAutoDelta { get; set; }
    }

    public class BarcodeField : Field
    {
        public BarcodeField()
        {
        }

        public override string OutputToXML()
        {
            return "";
        }
    }

    public class CidField : Field
    {
        public enum justification { left, right, center };

        public CidField()
        {
            just = justification.left;
            visible = true;
            font_name = "Arial";
            font_size = 6;
            font_bold = false;
            font_italic = false;
            rotation = "0";
        }

        public override string OutputToXML()
        {
            // For now, maybe add later.....
            return "";
        }

        public justification just;
        public bool visible;
        public String font_name;
        public UInt32 font_size;
        public bool font_bold;
        public bool font_italic;
        public string rotation;
    }

    public class TextField : Field
    {
        public enum justification {left, right, center, dec, dec_np };
        public enum text_case { upper, lower, mixed };
        public enum vert_align { bottom, baseline };

        public TextField()
        {
            just = justification.left;
	        va = vert_align.bottom;
	        tc = text_case.mixed;
	        multiline = false;
	        visible = true;
	        editable = true;
            insert_commas = true;
	        commas_locked = false;
	        negs_in_parens = true;
	        negs_locked = false;
            //round_calcs = false;
	        //round_locked = false;
	        link_page = 0;
	        font_locked = false;
	        font_name = "Courier New";
            font_size = 12;
	        font_bold = false;
	        font_italic = false;
	        //strict_validation = false;
	        validation = "";
	        validation_description = "";
            checkbox = false;
        }

        public override string OutputToXML()
        {
            string output = "";

            output += "<text ref=\""  + ref_name + "\"";

            switch (just)
            {
            case justification.right:
                output += " justification=\"right\"";
                break;
            case justification.center:
                output += " justification=\"center\"";
                break;
            case justification.dec:
                output += " justification=\"decimal\"";
                break;
            case justification.dec_np:
                output += " justification=\"decimal_no_point\"";
                break;
            case justification.left:
            default:
                output += " justification=\"left\"";
                break;
            }

            if (va != vert_align.bottom)
                output += " vertical_alignment=\"baseline\"";

            if (multiline)
                output += " multiline=\"true\"";

	        if (checkbox)
		        output += " checkbox=\"true\"";

	        //if (override_mask)
		     //   out << " override_mask=\"true\"";

            if (!visible)
                output += " visable=\"false\"";

            if (!editable)
                output += " editable=\"false\"";

            if (tc != text_case.mixed)
                output += " case=\"" + (tc == text_case.upper ? "upper" : "lower") + "\"";

            //if (link_page != 0)
                //    out << " link=\"" << link_page << "\"";
            if (!insert_commas || commas_locked)
            {
                if (insert_commas)
                    output += " insert_commas=\"true_locked\"";
                else
                {
                    if (commas_locked)
                        output += " insert_commas=\"false_locked\"";
                    else
                        output += " insert_commas=\"false\"";
                }
            }

            if (!negs_in_parens || negs_locked)
            {
                if (negs_in_parens)
                    output += " negatives_in_parenthesis=\"true_locked\"";
                else
                {
                    if (negs_locked)
                        output += " negatives_in_parenthesis=\"false_locked\"";
                    else
                        output += " negatives_in_parenthesis=\"false\"";
                }
            }

            output += ">\n";

            // Write the data element.
            if (data.Length == 0)
                output += "<data/>\n";
            //else if ( m_InputMask.Length() && !override_mask )
	        //{
		    //    if ( data == m_InputMask.RawToMaskedData(NULL) )
			//        out << "<data/>\n";
		    //    else
			//        out << "<data><![CDATA[" << data << "]]></data>\n";
	        //}
	        else
		        output += "<data><![CDATA[" + data + "]]></data>\n";
	
            // Write the bbox element.
            output += "<bbox left=\"" + bbox.left + "\" top=\"" +
                bbox.top + "\" right=\"" + bbox.right +
                "\" bottom=\"" + bbox.bottom + "\"/>\n";

            // Write the font element.
            output += "<font size=\"" + font_size + "\"";

            if (font_bold && font_italic)
                output += " style=\"bolditalic\"";
            else if (font_bold)
                output += " style=\"bold\"";
            else if (font_italic)
                output += " style=\"italic\"";
            else
                output += " style=\"regular\"";
            if (font_locked)
                output += " locked=\"true\"";
            //if ( !font_locked && font_autoshrink )
            //    out << " autoshrink=\"true\"";
            //if ( !font_locked && font_autoshrink && font_shrink_size > 0 )
            //    out << " shrink_size=\"" << font_shrink_size << "\"";
            output += ">" + font_name + "</font>\n";

            // Write the validation.
            if (validation.Length > 0)
            {
                output += "<validation>";
                output += "<description><![CDATA[" + validation_description + "]]></description>";
                output += "<regex><![CDATA[" + validation + "]]></regex></validation>\n";
            }
            // Write the epilog.
            output += "</text>\n";

            return output;
        }

        public justification just;
        public text_case tc;
        public vert_align va;
        public bool visible;
        public bool editable;
        public bool multiline;
        public bool insert_commas;
        public bool commas_locked;
        public bool negs_in_parens;
        public bool negs_locked;
        public UInt32 link_page;
        public String font_name;
        public bool font_locked;
        public UInt32 font_size;
        public bool font_bold;
        public bool font_italic;
        public bool checkbox;
        public String validation;
        public String validation_description;
    }
}
